﻿<?php
/**
 ***********************************************************************************************
 * Role updater plugin for Admidio
 *
 * Version 1.1 - Compatible with Admidio version 3.1
 *
 * With the help of this plugin you can easily create mailing lists from
 * checkbox fields in the user data.
 * 
 * Licensed under the MIT License (MIT)
 *
 * Copyright (c) 2016 - Matthias Jentsch - info@matthias-jentsch.de
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 ***********************************************************************************************
 */

// create path to plugin
$plugin_folder_pos = strpos(__FILE__, 'adm_plugins') + 11;
$plugin_file_pos   = strpos(__FILE__, 'role_updater.php');
$plugin_folder     = substr(__FILE__, $plugin_folder_pos+1, $plugin_file_pos-$plugin_folder_pos-2);

// include the global variables
if(!defined('PLUGIN_PATH'))
{
    define('PLUGIN_PATH', substr(__FILE__, 0, $plugin_folder_pos));
}
require_once(PLUGIN_PATH. '/../adm_program/system/common.php');

// include the language file and the configuration of the plugin
$gL10n->addLanguagePath(PLUGIN_PATH. '/'.$plugin_folder.'/languages');
require_once(PLUGIN_PATH. '/'.$plugin_folder.'/config.php');

// create a simple html page with some text
$page = new HtmlPage();
$page->addJavascriptFile($g_root_path.'/adm_program/libs/jquery/jquery.min.js');
$page->setHeadline($gL10n->get('PLG_ROLEUPDATER_HEADLINE'));
$page->addHtml('<strong>'.$gL10n->get('PLG_ROLEUPDATER_WELCOME').'</strong><br />');

// get all mailing lists
$mailingListRoles = $gDb->query('SELECT rol_name, rol_description, rol_id FROM '.TBL_CATEGORIES.' INNER JOIN '.TBL_ROLES.' ON cat_id = rol_cat_id WHERE cat_type=\'ROL\' AND cat_name=\''.$plg_roleupdater_maling_list_category.'\' ORDER BY rol_name');
while ($mailingListRole = $mailingListRoles->fetch())
{
	$page->addHtml($mailingListRole['rol_name'].': ');
	
	// in simple mode the description is empty; that means that members which have the checkbox field
	// with the rol_name has set to 1 should be assigned to the role
	$errorInDescription = false;
	if (is_null($mailingListRole['rol_description']))
	{
		// simple mode
		$negativeCondition = 'usf_name = \''.$mailingListRole['rol_name'].'\' AND usd_value <> \'1\'';
		$positiveCondition = 'usf_name = \''.$mailingListRole['rol_name'].'\' AND usd_value = \'1\'';
	}
	else
	{
		// advanced mode => parse the description
		// example for operating in AND mode: Year of birth = 1970 && Children >= 1 
		// example for operating in OR mode: Year of birth = 1970 || Married = 1
		$found = preg_match_all('/([^><=!]*)([><=!]*)([^&\|]*)([&\|]*)/', $mailingListRole['rol_description'], $matches);
		if ($found)
		{
			$conditionCount = count($matches[0]) - 1;
			$linkOperator = $matches[4][0];
		
			for($i=0; $i < $conditionCount; $i++)
			{
				switch ($matches[2][$i])
				{
				case '=':
					$opposite = '<>';
					break;
				case '!=':
				case '<>':
					$opposite = '=';
					break;
				case '>':
					$opposite = '<=';
					break;
				case '<':
					$opposite = '>=';
					break;
				case '>=':
					$opposite = '<';
					break;
				case '<=':
					$opposite = '>';
					break;
				default:
					$errorInDescription = true;
					$page->addHtml('<strong>'.$gL10n->get('PLG_ROLEUPDATER_DESCRIPTION_ERROR_COMPARISON').'</strong><br />');
					break;
				}
				$negativeCondition = $negativeCondition.'(usf_name = \''.addslashes(trim($matches[1][$i])).'\' AND usd_value '.$opposite.' \''.addslashes(trim($matches[3][$i])).'\')';
				$positiveCondition = $positiveCondition.'(usf_name = \''.addslashes(trim($matches[1][$i])).'\' AND usd_value '.$matches[2][$i].' \''.addslashes(trim($matches[3][$i])).'\')';
				switch ($matches[4][$i])
				{
				case '&&':
				case '||':
					if ($linkOperator !== $matches[4][$i])
					{
						$errorInDescription = true;
						$page->addHtml('<strong>'.$gL10n->get('PLG_ROLEUPDATER_DESCRIPTION_ERROR_MIXED').'</strong><br />');
						break;
					}					
					$negativeCondition = $negativeCondition.' OR ';
					$positiveCondition = $positiveCondition.' OR ';
					break;
				}
			}
		}
		else
		{
			$errorInDescription = true;
			$page->addHtml('<strong>'.$gL10n->get('PLG_ROLEUPDATER_DESCRIPTION_ERROR_REGEX').'</strong><br />');
		}
	}
	
	// don't process if an error was found
	if ($errorInDescription)
	{
		continue;
	}
	
	// delete members from the role
	if ($linkOperator !== '||')
	{
		// for simple mode or when in advanced mode when all conditions should met (AND)
		$delete = $gDb->query('DELETE FROM '.TBL_MEMBERS.' WHERE mem_rol_id = '.$mailingListRole['rol_id'].' AND mem_usr_id IN (SELECT usd_usr_id FROM '.TBL_USER_FIELDS.' INNER JOIN '.TBL_USER_DATA.' ON usf_id = usd_usf_id WHERE '.$negativeCondition.')');
	}
	else
	{
		// for advanced mode when only one condition should met (OR)
		$delete = $gDb->query('DELETE FROM '.TBL_MEMBERS.' WHERE mem_rol_id = '.$mailingListRole['rol_id'].' AND mem_usr_id IN (SELECT usd_usr_id FROM '.TBL_USER_FIELDS.' INNER JOIN '.TBL_USER_DATA.' ON usf_id = usd_usf_id WHERE '.$negativeCondition.' GROUP BY usd_usr_id HAVING COUNT(*) = '.$conditionCount.')');
	}
	$page->addHtml($delete->rowCount().' '.$gL10n->get('PLG_ROLEUPDATER_DELETED').' ');
	
	// select missing
	if ($linkOperator !== '&&')
	{
		// for simple mode or when in advanced mode when only one condition should met (OR)
		$missing = $gDb->query('SELECT DISTINCT usd_usr_id FROM '.TBL_USER_FIELDS.' INNER JOIN '.TBL_USER_DATA.' ON usf_id = usd_usf_id LEFT JOIN '.TBL_MEMBERS.' ON usd_usr_id = mem_usr_id AND mem_rol_id = '.$mailingListRole['rol_id'].' WHERE ('.$positiveCondition.') AND mem_rol_id IS NULL');
	}
	else
	{
		// for advanced mode when when all conditions should met (AND)
		$missing = $gDb->query('SELECT usd_usr_id FROM '.TBL_USER_FIELDS.' INNER JOIN '.TBL_USER_DATA.' ON usf_id = usd_usf_id LEFT JOIN '.TBL_MEMBERS.' ON usd_usr_id = mem_usr_id AND mem_rol_id = '.$mailingListRole['rol_id'].' WHERE ('.$positiveCondition.') AND mem_rol_id IS NULL GROUP BY usd_usr_id HAVING COUNT(*) = '.$conditionCount);
	}
	if ($missing->rowCount() === 0)
	{
		$page->addHtml('0 '.$gL10n->get('PLG_ROLEUPDATER_ADDED').'<br />');
	}
	else
	{
		$first = true;
		$insertStatement = 'INSERT INTO '.TBL_MEMBERS.' (mem_rol_id, mem_usr_id, mem_begin, mem_end, mem_leader, mem_usr_id_create, mem_timestamp_create, mem_count_guests) VALUES ';
		while ($userId = $missing->fetch())
		{
			if ($first === false)
			{
				$insertStatement = $insertStatement.', ';
			}
			else
			{
				$first = false;
			}
			$insertStatement = $insertStatement.'('.$mailingListRole['rol_id'].', '.$userId['usd_usr_id'].', \''.date('Y-m-d').'\', \'9999-12-31\', 0, 1, \''.date('Y-m-d H:i:s').'\', 0)';
		}
		$insert = $gDb->query($insertStatement);
		$page->addHtml($insert->rowCount().' '.$gL10n->get('PLG_ROLEUPDATER_ADDED').'<br />');
	}
}
$page->addHtml('<strong>'.$gL10n->get('PLG_ROLEUPDATER_GOODBYE').'</strong>');
$page->show();
?>